/**********************************************************************
*                                                                     *
*   Associative Coder of Buyanovsky (ACB)                             *
*                                                                     *
* triplet.h                                                           *
*                                                                     *
* Implemented by Martin Decky, distributed under GNU GPL              *
*                                                                     *
**********************************************************************/

#ifndef __TRIPLET_H__
#define __TRIPLET_H__

#include <iostream>

namespace ACB {

	class TripletEncoder {
		public:
			TripletEncoder(std::ostream & _outs);
			virtual ~TripletEncoder();
			
			virtual void Encode(const size_t distance, const bool sign, const size_t match, const char next, const bool complete);
			virtual void Flush(void);
		private:
			typedef unsigned char buftype;
		
			std::ostream * outs;
			buftype * bitbuf;
			size_t itemsize;
			size_t bufsize;
			size_t bufpos;
			
			void PushBits(const buftype * bits, const size_t cnt);
	};
	
	class TripletDecoder {
		public:
			TripletDecoder(std::istream & _ins);
			virtual ~TripletDecoder();
			
			virtual bool Decode(size_t & distance, bool & sign, size_t & match, char & next, bool & complete);
			virtual void Fill(void);
		private:
			typedef unsigned char buftype;
		
			std::istream * ins;
			buftype * bitbuf;
			size_t itemsize;
			size_t bufsize;
			size_t buflen;
			size_t bufpos;
			
			void PopBits(buftype * bits, const size_t cnt);
	};

}

#endif /* __TRIPLET_H__ */
