/**********************************************************************
*                                                                     *
*   Associative Coder of Buyanovsky (ACB)                             *
*                                                                     *
* vocabulary.h                                                        *
*                                                                     *
* Implemented by Martin Decky, distributed under GNU GPL              *
*                                                                     *
**********************************************************************/

#ifndef __VOCABULARY_H__
#define __VOCABULARY_H__

#include <iostream>
#include <list>

namespace ACB {

	class Vocabulary {
		public:
			Vocabulary();
			virtual ~Vocabulary();
			
			virtual bool Exhausted(void) const;
			virtual void Input(const char * input, const size_t cnt);
			virtual void Compare(size_t & distance, bool & sign, size_t & match, char & next, bool & complete);
			virtual char * GetContent(const size_t distance, const bool sign, const size_t match, const char next, const bool complete, size_t & size);
		private:
			typedef std::list<size_t> VocList;
			typedef VocList::iterator iterator;
		
			char * buffer;
			size_t bufsize;
			size_t sbuf;
			VocList list;
			
			size_t ContextMatch(void);
			size_t ContentMatch(const size_t shift, char & next, bool & complete) const;
			bool ContextCompare(const size_t shift) const;
			void UpdateVocabulary(const size_t cnt);
			void DebugPrint(void);
	};

}

#endif /* __VOCABULARY_H__ */
